@@ -21,6 +21,10 @@ func aptlyPackageSearch(cmd *commander.Command, args []string) error {
 	}
 
 	result := q.Query(context.CollectionFactory().PackageCollection())
+	if result.Len() == 0 {
+		return fmt.Errorf("no results")
+	}
+
 	result.ForEach(func(p *deb.Package) error {
 		context.Progress().Printf("%s\n", p)
 		return nil
