@@ -37,3 +37,13 @@ class SearchSnapshot4Test(BaseTest):
     fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
     outputMatchPrepare = lambda _, s: "\n".join(sorted(s.split("\n")))
     runCmd = "aptly snapshot search -with-deps wheezy-main 'Name (nginx)'"
+
+
+class SearchSnapshot5Test(BaseTest):
+    """
+    search snapshot: no results
+    """
+    fixtureDB = True
+    fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
+    runCmd = "aptly snapshot search -with-deps wheezy-main 'Name (no-such-package)'"
+    expectedCode = 1
